unit Cfgspect;

interface

uses sysutils,forms,classes,iniFiles,adPort,dialogs,
     jmmUtil, oomisc;

const
  NomSection = 'IC6300';

type
  strCourte = string[20];
  TJenway = (j6300,j6305,j6310,j6315,j6320,genova);
  TmodeRegressi = (rFichier,rColonne,rPage);
  TEtatSpectre = (esBlancEnCours,esSpectreEnCours,esAttente,esTest,esInit,esAbsorbance,esEcho);

  Tspectro = class(Tobject)
      constructor Create;
      function GetTimeOutLambda : integer;
      function getGetAbsLambda : strCourte;
      function getGotoLambda : strCourte;
      function getSetZero : strCourte;
      public
        MiniAbs,MaxiAbs : double;
        MaxiLambda  : integer; // en nm
        DebutLambda,FinLambda : integer;
        NbreDecAbs : integer;
        PasLambdaMax,pasLambda : integer;

        ComNumberS : word;
        Baud     : LongInt;
        Parite   : TParity;
        DataBits : word;
        StopBits : word;

        Jenway : tJenway;

        FinCommande : char;
        AbsCourante : double;
        LambdaCourante,LambdaDemandee : integer;

        valeurRouleau : array[boolean] of strCourte;
        TimeOutAbs : integer;
        Rouleau : boolean;
        CompteurLambda : integer;
        stable,sature,faireGOTO : boolean;
        AbsBlanc : vecteur;
        property setZero : strCourte read getSetZero;
        property GetAbsLambda : strCourte read getgetAbsLambda;
        property GotoLambda : strCourte read getGOTOLambda;

        property TimeOutLambda : integer read getTimeOutLambda;
        Function BlancFait : boolean;
        Function NomConstructeur : strCourte;
        Function MiniLambda  : integer;
        Procedure SetAbsLambda(chaineLue : shortString);
        Procedure SetAbsorbance(chaineLue : shortString);
        function StrAbs(valeur : double) : shortString;
        procedure resetAbs;
        function IsFiltreUV : boolean;
        procedure lit(NomFichier : string);
        procedure ecrit(NomFichier : string);
  end;

var spectro : TSpectro;

const
   filtres : array[Tjenway,1..2] of integer =
       ((360,1200),(391,551),(391,551),(391,551),(360,1200),(360,1200)); // 1200 en dehors

implementation

const
NomSpectro : array[Tjenway] of strCourte =
  ('Jenway 6300','Jenway 6305','Jenway 6310','Jenway 6315','Jenway 6320','Genova');
MiniL : array[Tjenway] of integer =
  (320,198,320,198,320,320);

constructor TSpectro.Create;
begin
inherited create;
Jenway := j6300;
NbreDecAbs := 3;
MiniAbs := -0.3;
MaxiAbs := 2;
MaxiLambda := 1000;
ComNumberS := 1;
Baud := 1200;
PasLambda := 2;
PasLambdaMax := 16;
Parite := pOdd;
DataBits := 7;
StopBits := 1;
FinCommande := crCR;
TimeOutAbs := 250;
end;

Function Tspectro.getGetAbsLambda : strCourte;
begin
   if jenway in [j6315,j6310] then result := 'ABS' else result := 'A';
end;

Function Tspectro.getGotoLambda : strCourte;
begin
   if jenway in [j6315,j6310] then result := 'GOTO' else result := 'G';
end;

Function Tspectro.getSetZero : strCourte;
begin
   if jenway in [j6315,j6310] then result := 'ZERO' else result := 'Z';
end;

Function Tspectro.NomConstructeur : strCourte;
begin
     result := nomSpectro[jenway]
end;

Function Tspectro.MiniLambda : integer;
begin
    result := MiniL[jenway]
end;

procedure chargeAppareil;
begin
    spectro := TSpectro.Create;
    spectro.Lit(ParamStr(0));
end;

Function Tspectro.BlancFait : boolean;
var i : integer;
begin
     result := false;
     for i := debutLambda to finLambda do
          if absBlanc[i]=sansSignif then exit;
     result := true;
end;

procedure TSpectro.lit(NomFichier : string);
var i : integer;
    Ini : TmeminiFile;
begin
    NomFichier := ChangeFileExt(NomFichier,'.INI');
    Ini := TmeminiFile.create(NomFichier);
    ComNumberS := Ini.ReadInteger(nomSection,'Com',1);
    PasLambda := Ini.ReadInteger(nomSection,'PasLambda',2);
    Jenway := TJenway(Ini.ReadInteger(nomSection,'63xx',ord(j6300)));
    debutLambda := Ini.ReadInteger(nomSection,'Debut',debutLambda);
    finLambda := Ini.ReadInteger(nomSection,'Fin',finLambda);
    for i := miniLambda to maxiLambda do
        absBlanc[i] := Ini.ReadFloat('LigneBase',intToStr(i),sansSignif);
    Ini.free;
end;

procedure SauveAppareil;
begin
    spectro.ecrit(ParamStr(0))
end;

procedure Tspectro.ecrit(NomFichier : string);
var Ini : TmeminiFile;
    i : integer;
begin
    NomFichier := ChangeFileExt(NomFichier,'.INI');
    try
    Ini := TmeminiFile.create(nomFichier);
    Ini.WriteInteger(nomSection,'Com',ComNumberS);
    Ini.WriteFloat(nomSection,'PasLambda',pasLambda);
    Ini.WriteInteger(nomSection,'Debut',debutLambda);
    Ini.WriteInteger(nomSection,'63xx',ord(jenway));
    Ini.WriteInteger(nomSection,'Fin',finLambda);
    for i := miniLambda to maxiLambda do
        Ini.WriteFloat('LigneBase',intToStr(i),absBlanc[i]);
    ini.UpdateFile;
    ini.free;
    except
    end;
end;

Procedure TSpectro.setAbsLambda(chaineLue : shortString);
var posTab : integer;
begin
    posTab := pos(crTab,chaineLue);
    ValeurRouleau[rouleau] := copy(chaineLue,1,posTab-1);
    try
    AbsCourante := strToFloatWin(ValeurRouleau[rouleau]);
    sature := (AbsCourante<(-0.3)) or (AbsCourante>2.5);
    except
       absCourante := 0;
    end;
    if absCourante>2*MaxiAbs then absCourante := 2*MaxiAbs;
    if absCourante<2*MiniAbs then absCourante := 2*MiniAbs;
    try
    LambdaCourante := strToInt(copy(chaineLue,posTab+1,length(chaineLue)-posTab));
    except
       LambdaCourante := 0;
    end;
    if lambdaCourante<MiniLambda then LambdaCourante := 0;
    if lambdaCourante>MaxiLambda then LambdaCourante := 0;
    if lambdaCourante=lambdaDemandee
       then begin
          rouleau := not rouleau;
          faireGOTO := false;
          stable := valeurRouleau[true]<>'';
          if stable then case valeurRouleau[true][1] of
             '2' : stable := copy(valeurRouleau[true],1,3)=copy(valeurRouleau[false],1,3); // 2.1
             '1' : stable := copy(valeurRouleau[true],1,4)=copy(valeurRouleau[false],1,4); // 1.23
             '0' : stable := copy(valeurRouleau[true],1,5)=copy(valeurRouleau[false],1,5); // 0.231
             '-' : stable := copy(valeurRouleau[true],1,5)=copy(valeurRouleau[false],1,5); // -0.23
             else stable := true;
          end;
          if Jenway=j6320 then stable := true;        
       end
       else begin
            stable := false;
            inc(compteurLambda);
            faireGOTO := compteurLambda>5;
            valeurRouleau[false] := '';
            valeurRouleau[true] := '';            
       end;
end;

Procedure TSpectro.setAbsorbance(chaineLue : shortString);
var posBlanc : integer;
begin
    posBlanc := pos(' ',chaineLue);
    while posBlanc>0 do begin
          delete(chaineLue,posBlanc,1);
          posBlanc := pos(' ',chaineLue);
    end;
    posBlanc := pos(crCR,chaineLue);
    while posBlanc>0 do begin
          delete(chaineLue,posBlanc,1);
          posBlanc := pos(crCR,chaineLue);
    end;
    posBlanc := pos(crTab,chaineLue);
    try
    AbsCourante := strToFloatWin(copy(chaineLue,1,posBlanc-1));
    except
       absCourante := 0;
    end;
    if absCourante>2*MaxiAbs then absCourante := 2*MaxiAbs;
    if absCourante<2*MiniAbs then absCourante := 2*MiniAbs;
end;

function Tspectro.StrAbs(valeur : double) : shortString;
begin
    if valeur>2*MaxiAbs then valeur := 2*MaxiAbs;
    if valeur<2*MiniAbs then valeur := 2*MiniAbs;
    if valeur=sansSignif
        then result := '0'
        else result := FloatToStrF(valeur,ffFixed,NbreDecAbs+2,NbreDecAbs)
end;

procedure Tspectro.resetAbs;
begin
    compteurLambda := 0;
    faireGOTO := false;
    stable := false;
    ValeurRouleau[false] := '';
    ValeurRouleau[true] := '';    
end;

function Tspectro.IsFiltreUV : boolean;
begin
       result := ((lambdaDemandee-filtres[jenway,1])<pasLambda) or
                 ((lambdaDemandee-filtres[jenway,2])<pasLambda);
end;

function Tspectro.GetTimeOutLambda : integer;
begin
  if Jenway=j6320
     then result := 2500
     else result := 1200;
end;

initialization
    chargeAppareil

finalization
    sauveAppareil
end.
