UNIT Chromato;

// Valeurs des couleurs pour la reprsentation du spectre de la lumire blanche
//  380 nm < lambda < 780 nm

INTERFACE

CONST   lambda_min = 380;    { nm }
        lambda_max = 780;

Function lambda_color(lambda : integer) : longint;

IMPLEMENTATION


CONST resolution = 5.0;
      aR = 1.0622;
      aG = 1.0000;
      aB = 1.7826;

TYPE  data = ARRAY[-1..82] OF DOUBLE;

VAR cR, cG, cB : data;

Function lambda_color(lambda : integer) : longInt;
var num : integer;
    x : double;

 FUNCTION accent(alpha : DOUBLE) : byte;
 BEGIN
   IF alpha>0
      THEN if alpha<1
           then accent := trunc(255*alpha)
           else accent := 255
      ELSE accent := 0
 END;

 FUNCTION interpole(VAR cc : data) : DOUBLE ;
 VAR a, b, c, d : DOUBLE ;
 BEGIN
   a := (cc[num + 2] + 3 * cc[num] - 3 * cc[num + 1] - cc[num - 1]) / 2 ;
   b := (4 * cc[num + 1] + 2 * cc[num - 1] - cc[num + 2] - 5 * cc[num]) / 2 ;
   c := (cc[num + 1] - cc[num - 1]) / 2 ;
   d := cc[num] ;
   interpole := ((a * x + b) * x + c) * x + d ;
 END;

 VAR rouge,vert,bleu : byte;
 BEGIN
  IF(lambda >= lambda_Max) or (lambda <= lambda_Min)
      THEN begin
         rouge := 0;
         vert := 0;
         bleu := 0;
      end
      ELSE begin
         x := (lambda - lambda_Min)/resolution;
         num := trunc(x);
         x := frac(x);
         rouge := accent(interpole(cR)/aR);
         vert := accent(interpole(cG)/aG);
         bleu := accent(interpole(cB)/aB);
      end;
   result := bleu*$10000+vert*$100+rouge;
 END ;

Procedure Init;
 BEGIN
   cR[-1] := 0.0000 ; cG[-1] := 0.0000 ; cB[-1] := 0.0000 ;
   cR[ 0] := 0.0014 ; cG[ 0] := 0.0000 ; cB[ 0] := 0.0065 ;
   cR[ 1] := 0.0022 ; cG[ 1] := 0.0001 ; cB[ 1] := 0.0105 ;
   cR[ 2] := 0.0042 ; cG[ 2] := 0.0001 ; cB[ 2] := 0.0201 ;
   cR[ 3] := 0.0076 ; cG[ 3] := 0.0002 ; cB[ 3] := 0.0362 ;
   cR[ 4] := 0.0143 ; cG[ 4] := 0.0004 ; cB[ 4] := 0.0629 ;

   cR[ 5] := 0.0232 ; cG[ 5] := 0.0006 ; cB[ 5] := 0.1102 ;
   cR[ 6] := 0.0435 ; cG[ 6] := 0.0012 ; cB[ 6] := 0.2074 ;
   cR[ 7] := 0.0776 ; cG[ 7] := 0.0022 ; cB[ 7] := 0.3713 ;
   cR[ 8] := 0.1344 ; cG[ 8] := 0.0040 ; cB[ 8] := 0.6456 ;
   cR[ 9] := 0.2148 ; cG[ 9] := 0.0073 ; cB[ 9] := 1.0391 ;

   cR[10] := 0.2839 ; cG[10] := 0.0116 ; cB[10] := 1.3856 ;
   cR[11] := 0.3285 ; cG[11] := 0.0168 ; cB[11] := 1.6230 ;
   cR[12] := 0.3483 ; cG[12] := 0.0230 ; cB[12] := 1.7471 ;
   cR[13] := 0.3481 ; cG[13] := 0.0298 ; cB[13] := 1.7826 ;
   cR[14] := 0.3362 ; cG[14] := 0.0380 ; cB[14] := 1.7721 ;

   cR[15] := 0.3187 ; cG[15] := 0.0480 ; cB[15] := 1.7441 ;
   cR[16] := 0.2908 ; cG[16] := 0.0600 ; cB[16] := 1.6692 ;
   cR[17] := 0.2511 ; cG[17] := 0.0739 ; cB[17] := 1.5281 ;
   cR[18] := 0.1954 ; cG[18] := 0.0910 ; cB[18] := 1.2876 ;
   cR[19] := 0.1421 ; cG[19] := 0.1126 ; cB[19] := 1.0419 ;

   cR[20] := 0.0956 ; cG[20] := 0.1390 ; cB[20] := 0.8130 ;
   cR[21] := 0.0580 ; cG[21] := 0.1693 ; cB[21] := 0.6162 ;
   cR[22] := 0.0320 ; cG[22] := 0.2080 ; cB[22] := 0.4652 ;
   cR[23] := 0.0147 ; cG[23] := 0.2586 ; cB[23] := 0.3533 ;
   cR[24] := 0.0049 ; cG[24] := 0.3230 ; cB[24] := 0.2720 ;

   cR[25] := 0.0024 ; cG[25] := 0.4073 ; cB[25] := 0.2023 ;
   cR[26] := 0.0093 ; cG[26] := 0.5030 ; cB[26] := 0.1482 ;
   cR[27] := 0.0291 ; cG[27] := 0.6082 ; cB[27] := 0.1082 ;
   cR[28] := 0.0633 ; cG[28] := 0.7100 ; cB[28] := 0.0782 ;
   cR[29] := 0.1096 ; cG[29] := 0.7932 ; cB[29] := 0.0573 ;

   cR[30] := 0.1655 ; cG[30] := 0.8620 ; cB[30] := 0.0422 ;
   cR[31] := 0.2257 ; cG[31] := 0.9149 ; cB[31] := 0.0298 ;
   cR[32] := 0.2904 ; cG[32] := 0.9540 ; cB[32] := 0.0203 ;
   cR[33] := 0.3597 ; cG[33] := 0.9803 ; cB[33] := 0.0134 ;
   cR[34] := 0.4334 ; cG[34] := 0.9950 ; cB[34] := 0.0087 ;

   cR[35] := 0.5121 ; cG[35] := 1.0000 ; cB[35] := 0.0057 ;
   cR[36] := 0.5945 ; cG[36] := 0.9950 ; cB[36] := 0.0039 ;
   cR[37] := 0.6784 ; cG[37] := 0.9786 ; cB[37] := 0.0027 ;
   cR[38] := 0.7621 ; cG[38] := 0.9520 ; cB[38] := 0.0021 ;
   cR[39] := 0.8425 ; cG[39] := 0.9154 ; cB[39] := 0.0018 ;

   cR[40] := 0.9163 ; cG[40] := 0.8700 ; cB[40] := 0.0017 ;
   cR[41] := 0.9786 ; cG[41] := 0.8163 ; cB[41] := 0.0014 ;
   cR[42] := 1.0263 ; cG[42] := 0.7570 ; cB[42] := 0.0011 ;
   cR[43] := 1.0567 ; cG[43] := 0.6949 ; cB[43] := 0.0010 ;
   cR[44] := 1.0622 ; cG[44] := 0.6310 ; cB[44] := 0.0008 ;

   cR[45] := 1.0456 ; cG[45] := 0.5668 ; cB[45] := 0.0006 ;
   cR[46] := 1.0026 ; cG[46] := 0.5030 ; cB[46] := 0.0003 ;
   cR[47] := 0.9384 ; cG[47] := 0.4412 ; cB[47] := 0.0002 ;
   cR[48] := 0.8544 ; cG[48] := 0.3810 ; cB[48] := 0.0002 ;
   cR[49] := 0.7514 ; cG[49] := 0.3210 ; cB[49] := 0.0001 ;

   cR[50] := 0.6424 ; cG[50] := 0.2650 ; cB[50] := 0.0000 ;
   cR[51] := 0.5419 ; cG[51] := 0.2170 ; cB[51] := 0.0000 ;
   cR[52] := 0.4479 ; cG[52] := 0.1750 ; cB[52] := 0.0000 ;
   cR[53] := 0.3608 ; cG[53] := 0.1382 ; cB[53] := 0.0000 ;
   cR[54] := 0.2835 ; cG[54] := 0.1070 ; cB[54] := 0.0000 ;

   cR[55] := 0.2187 ; cG[55] := 0.0816 ; cB[55] := 0.0000 ;
   cR[56] := 0.1649 ; cG[56] := 0.0610 ; cB[56] := 0.0000 ;
   cR[57] := 0.1212 ; cG[57] := 0.0446 ; cB[57] := 0.0000 ;
   cR[58] := 0.0874 ; cG[58] := 0.0320 ; cB[58] := 0.0000 ;
   cR[59] := 0.0636 ; cG[59] := 0.0232 ; cB[59] := 0.0000 ;

   cR[60] := 0.0468 ; cG[60] := 0.0170 ; cB[60] := 0.0000 ;
   cR[61] := 0.0329 ; cG[61] := 0.0119 ; cB[61] := 0.0000 ;
   cR[62] := 0.0227 ; cG[62] := 0.0082 ; cB[62] := 0.0000 ;
   cR[63] := 0.0158 ; cG[63] := 0.0057 ; cB[63] := 0.0000 ;
   cR[64] := 0.0114 ; cG[64] := 0.0041 ; cB[64] := 0.0000 ;

   cR[65] := 0.0081 ; cG[65] := 0.0029 ; cB[65] := 0.0000 ;
   cR[66] := 0.0058 ; cG[66] := 0.0021 ; cB[66] := 0.0000 ;
   cR[67] := 0.0041 ; cG[67] := 0.0015 ; cB[67] := 0.0000 ;
   cR[68] := 0.0029 ; cG[68] := 0.0010 ; cB[68] := 0.0000 ;
   cR[69] := 0.0020 ; cG[69] := 0.0007 ; cB[69] := 0.0000 ;

   cR[70] := 0.0014 ; cG[70] := 0.0005 ; cB[70] := 0.0000 ;
   cR[71] := 0.0010 ; cG[71] := 0.0004 ; cB[71] := 0.0000 ;
   cR[72] := 0.0007 ; cG[72] := 0.0002 ; cB[72] := 0.0000 ;
   cR[73] := 0.0005 ; cG[73] := 0.0002 ; cB[73] := 0.0000 ;
   cR[74] := 0.0003 ; cG[74] := 0.0001 ; cB[74] := 0.0000 ;

   cR[75] := 0.0002 ; cG[75] := 0.0001 ; cB[75] := 0.0000 ;
   cR[76] := 0.0002 ; cG[76] := 0.0001 ; cB[76] := 0.0000 ;
   cR[77] := 0.0001 ; cG[77] := 0.0000 ; cB[77] := 0.0000 ;
   cR[78] := 0.0001 ; cG[78] := 0.0000 ; cB[78] := 0.0000 ;
   cR[79] := 0.0001 ; cG[79] := 0.0000 ; cB[79] := 0.0000 ;

   cR[80] := 0.0000 ; cG[80] := 0.0000 ; cB[80] := 0.0000 ;
   cR[81] := 0.0000 ; cG[81] := 0.0000 ; cB[81] := 0.0000 ;
   cR[82] := 0.0000 ; cG[82] := 0.0000 ; cB[82] := 0.0000 ;

end;

Initialization
Init

END.



