unit Jmmutil;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Dialogs, Forms,
  StdCtrls, Consts, controls, oomisc, AwUser, AwWin32, math, registry;

const
     MaxVecteur = 2048;
     MaxReal : double = 1e+35;
     MinReal : double = 1e-35;
     ProcheReal : double = 1e-6;
     SansSignif : double = 1e-26;

     UnJour = 24*3600;
     UneHeure = 3600;
     QuartdHeure = 15*60;
     UneMinute = 60;

     Precisiondouble = 1e-7;
     PrecisionTemps = 7; { Precision double pour FloatToStrF }
     PrecisionAbs = 4;
     LongUnite = 20;
     SymbReg : char = '';
     SymbReg2 : char = '&';
     Tabulation = #9;
     CaracCtrl = '|';
     CodeCtrl : array[#0..#31] of string[3] =
              ('NUL','SOH','STX','ETX','EOT','ENQ','ACK','BEL','BS','TAB',
               'LF','VT','FF','CR','SO','SI','DLE','DC1','DC2','DC3',
               'DC4','NAK','SYN','ETB','CAN','EM','SUB','ESC','FS','GS',
               'RS','US');


        alpha = #128;
        beta = #129;
        chi = #130;
        delta = #131;
        deltaMaj = #150;
        epsilon = #132;
        phi = #133; phiMaj = #151;
        gamma = #134; gammaMaj = #152;
        eta = #135;
        LettreLambda = #136; lambdaMaj = #153;
        nu = #138;
        piMin = #139;piMaj = #154;
        theta = #140;thetaMaj = #155;
        rho = #141;
        sigma = #142;sigmaMaj = #156;
        tau = #143;
        omega = #144;omegaMaj = #157;
        xi = #147;xiMaj = #158;
        psi = #148;psiMaj = #159;
        zeta = #149;
        mu = #181;
        vide = #248;videMaj = #216;

       ChiffreExp : array['0'..'9'] of char =
  (#186,#185,#178,#179,#165,#169,#172,#173,#174,#188);
       MoinsExp = #170;
       PlusExp = #184;

type
    IndiceVecteur = 0..MaxVecteur;
    Vecteur = array[IndiceVecteur] of double;

    TcharSet = set of char;
    strCourte = string[LongUnite];
    CoefUnite = (zero,yocto,zepto,atto,femto,pico,nano,micro,milli,
               nulle,kilo,mega,giga,tera,peta,exa,zetta,yotta,infini);

Const
    Majuscule   : TcharSet = ['A'..'Z'];
    Minuscule   : TcharSet = ['a'..'z'];
    Grecque  : TcharSet = [alpha,beta,gamma,gammaMaj,delta,deltaMaj,chi,
              epsilon,phi,phiMaj,eta,psi,psiMaj,LettreLambda,lambdaMaj,mu,nu,
              piMin,piMaj,theta,thetaMaj,rho,sigma,sigmaMaj,tau,omega,omegaMaj,
              xi,xiMaj,zeta];
    toutCarac   : TcharSet = [#032..#151,#224..#250];
    Chiffre     : TcharSet = ['0'..'9'];
    ChiffreEtc  : TcharSet = ['0'..'9','+','-','e','E','.',','];
    crTab = char(vk_tab);
    crSuppr = char(vk_clear);
    crSupprArr = char(vk_back);
    crHaut = char(vk_up);
    crBas = char(vk_down);
    crGauche = char(vk_left);
    crDroite = char(vk_right);
    crHome = char(vk_home);
    crEnd = char(vk_end);
    crCR = #13;
    crLF = #10;
    crESC = #27;
    crAck = #6;
    crBEL = #7;
    crETX = #3;
    crNoAck = #21;
    separateur : TCharSet = [' ',crTab,','];

    caracPrefixe : TcharSet =
	   ['y','z','a','f','p','n','','m','k','M','G','T','P','E'];

Function ConvertitChaine(nombre : strCourte;var valeur : double) : boolean;
Function FloatToStrPoint(valeur : double) : strCourte;
Function StrToFloatWin(nombre : shortString) : double;
Function FormatCourt(Nombre : double) : strCourte;
Function EncodeDuree(duree : LongInt) : TdateTime;
Function DureeEcoulee(InstantDebut : TdateTime) : longInt;
function Pad(s : shortString;Len : Byte) : shortString;

function IsPortAvailable(ComNum : Cardinal) : Boolean;

Function Dix(x : integer) : double;

Procedure GetMinMax(const V : vecteur;N : indiceVecteur;var mini,maxi : double);
Procedure VerifMinMaxReal(var mini,maxi : double);
Function DateTimeToFloat(nombre : strCourte) : integer;
function duree125(duree : double) : integer;

implementation

Function StrToFloatWin(nombre : shortString) : double;
var posSeparator : byte;
begin
   if decimalSeparator<>',' then begin
      posSeparator := pos(',',nombre);
      if posSeparator>0 then Nombre[posSeparator] := decimalSeparator;
   end
   else if decimalSeparator<>'.' then begin
      posSeparator := pos('.',nombre);
      if posSeparator>0 then Nombre[posSeparator] := decimalSeparator;
   end;
   try
   StrToFloatWin :=  strToFloat(nombre);
   except
   StrToFloatWin := sansSignif; 
   end;
end;

Function FloatToStrPoint(valeur : double) : strCourte;
var posSeparator : byte;
begin
   result := FloatToStrF(valeur,ffGeneral,7,2);
   if decimalSeparator<>'.' then begin
      posSeparator := pos(decimalSeparator,result);
      if posSeparator>0 then result[posSeparator] := '.';
   end;
end;

Function ConvertitChaine(nombre : strCourte;var valeur : double) : boolean;
begin
   if (upcase(nombre[1])='E')
	  then nombre := '1'+nombre
	  else if (nombre[1]='.') or (nombre[1]=',')
		   then nombre := '0'+nombre;
   valeur := StrToFloatWin(nombre);
   convertitChaine := valeur<>sansSignif;
end;

Function FormatCourt(Nombre : double) : strCourte;
var Nexp : integer;
begin
     if abs(Nombre)<1e-10
         then Nexp := 2
         else if abs(Nombre)<1e+10
              then Nexp := 1
              else Nexp := 2;
      FormatCourt := FloatToStrF(nombre,ffGeneral,PrecisionAbs,Nexp)
end;

Function FormatLong(Nombre : double) : strCourte;
var Nexp : integer;
begin
     if abs(Nombre)<1e-10
         then Nexp := 2
         else if abs(Nombre)<1e+10
              then Nexp := 1
              else Nexp := 2;
      FormatLong := FloatToStrF(nombre,ffGeneral,PrecisionTemps,Nexp)
end;

Function Skip(var s : shortString;carac : char) : shortString;
var j : integer;
begin
      j := pos(carac,s);
      while j<>0 do begin
         Delete(s,j,1);
         j := pos(carac,s)
      end;
end;

Function EncodeDuree(duree : LongInt) : TdateTime;
var h,m,s,ms : word;
begin
    h := duree div 3600;
    duree := duree-h*3600;
    m := duree div 60;
    duree := duree-m*60;
    s := duree;
    ms := 0;
    EncodeDuree := EncodeTime(h,m,s,ms);
end;

Function DureeEcoulee(InstantDebut : TdateTime) : longInt;
var h,m,s,ms : word;
begin
      decodeTime(time-instantDebut,h,m,s,ms);
      DureeEcoulee := h*3600+m*60+s;
end;

function PadChar(S : ShortString;Len : Byte;Achar : char) : ShortString;
{Return a string right-padded to length len with Achar}
var
  o : ShortString;
  SLen : Byte absolute S;
begin
   if SLen>Len then S := copy(S,1,Len);
   o[0] := Chr(Len);
   Move(S[1], o[1], SLen);
   FillChar(o[Succ(SLen)], Len-SLen,Achar);
   PadChar := o;
end;

function Pad(S : ShortString;Len : Byte) : ShortString;
{Return a string right-padded to length len with ' '}
begin
     result := PadChar(S,len,' ')
end;

function Dix(x : integer) : double;
Begin
    Dix := Power(10,X)
end;

Procedure GetMinMax(const V : vecteur;N : indiceVecteur;var mini,maxi : double);
var i : indiceVecteur;
begin
     Mini := V[0];
     Maxi := V[0];
     for i := 1 to pred(N) do
         if V[i]>Maxi
            then Maxi := V[i]
            else if V[i]<Mini
               then Mini := V[i]
end;

function IsPortAvailable(ComNum : Cardinal) : Boolean;

  function MakeComName(const Dest : PChar; const ComNum : Cardinal) : PChar;
    {-Return a string like 'COMXX'}
  begin
    StrFmt(Dest,'\\.\COM%d',[ComNum]);
    MakeComName := Dest;
  end;

var
  ComName : array[0..12] of Char;
  Res : Integer;
  DeviceLayer : TApdBaseDispatcher;
  Reg : Tregistry;
begin
  Reg := Tregistry.create;
  Reg.rootKey := Hkey_local_machine;
  Reg.openKey('\Hardware\DeviceMap\SerialComm',true);
  result := (Reg.readString('\Device\porte'+intToStr(comNum)))=''; // ports internes Toshiba ?
  Reg.Free;
  if not result then exit;
  DeviceLayer  := TApdWin32Dispatcher.Create(nil);
  try
    if (ComNum = 0) then
      Result := False
    else begin
      Res := DeviceLayer.OpenCom(MakeComName(ComName,ComNum), 64, 64);
      if (Res < 0) and 
         (GetLastError <> DWORD(Abs(ecAccessDenied)))
         then Result := False
      else begin
        Result := True;
        DeviceLayer.CloseCom;
      end;
    end;
  finally
    DeviceLayer.Free;
  end;
end;

Procedure VerifMinMaxReal(var mini,maxi : double);
var z : double;
begin
   if Maxi<Mini then begin
	  z := Maxi;
	  Maxi := Mini;
	  Mini := z;
   end;
end;

function PosN(const C : shortString;S : shortString;const K : Integer): Integer;
var
  P : Integer;
begin
  Result := 0;
  if K>0 then Delete(S,1,K);
  P := Pos(C, S);
  if P>0 then result := K+P;
end;

Procedure TrimComplet(var s : shortString);
var longueur,posCourant : byte;
begin
	longueur := Length(s);
	posCourant := 1;
	while (posCourant<=longueur) do
		if ord(s[posCourant])<=32
			then begin
			   Delete(s,posCourant,1);
			   dec(longueur);
			end
			else inc(posCourant);
end;

Function DateTimeToFloat(nombre : strCourte) : integer;
var i : integer;
begin
     result := 0;
     trimComplet(nombre);
     i:=pos(':',nombre);
     if i>0
        then begin
           i := posN(':',nombre,2);
           if i=0 then nombre := '0:'+nombre;
           try
           result := round(StrToTime(nombre)*60*60*24);
           except
           end;
        end; {i>0}
        i:=pos('h',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := result+StrToInt(copy(nombre,1,pred(i)))*60*60;
              except
              end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('min',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := result+round(StrToFloatWin(copy(nombre,1,pred(i)))*60);
              except
              end;
           end;
           delete(nombre,1,i+2);
        end;
        i:=pos('s',nombre);
        if i>0 then delete(nombre,i,1);
        if nombre<>'' then begin
            try
            result := result+round(strToFloatWin(nombre));
            except
            end;
        end;
        if result = 0 then result := 1;
end;

function duree125(duree : double) : integer;
var coeff : double;
    exposant,mantisse : integer;
begin
     duree := abs(duree);
     if duree<1e-6 then begin
        result := 0;
        exit;
     end;
     exposant := floor(log10(duree));
     coeff := power(10,exposant);
     mantisse := round(duree/coeff);
     if mantisse>2
        then if mantisse<6
             then mantisse := 5
             else mantisse := 10;
     coeff := mantisse*coeff;
     result := round(coeff);
end;

end.

