unit Graphker;

interface

uses winTypes, winProcs, sysUtils, graphics, classes, printers,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
     jmmutil, cfgspect, chromato, math;

const
        IntSansSignif = -1;
        LongNombreAxe = 4;
        NbreGraduations = 10;
        NbreGradMin = 5;
        CouleurGr = clBlack;
        CouleurCalc = clRed;
        CouleurTitre = clBlack;
        dimPoint = 4; 

type
  Enombre = class(Exception);
  Ttrace = (tLigne,tSpectre,tPoint,tCinet);
  Trepere = class
         Valeur : double;
         Text   : shortString;
  end;

	Tgraphe = class;

Tmonde = class
    mini,maxi : double;
	a,b : double; { entier = rel*a+b }
	zeroInclus : boolean;
    avecAxe,grandAxe : boolean;
	nom : strCourte;
    unite : strCourte;
    PosCurseur : integer;
    formatDuree : strCourte;
    arrondi : double;
    isDate : boolean;
    deltaAxe,pointDebut : double;
    signif : shortString;
{ deltaAxe = cart entre graduations sur les axes }
	constructor create;
    procedure setDelta;
    Procedure setDeltaDateTime;
    procedure affecteEntier(r : double;var i : integer);
    Function FormatAxe(x : double) : strCourte;
	end;

Tgraphe = class
           valeurY,valeurX : vecteur;
           Trace : Ttrace;
           NbrePoints : integer;
           NbreCourbe : integer;
           etatSpectre : TetatSpectre;
      	   mondeX,mondeY : Tmonde;
	       LimiteCourbe,LimiteFenetre : Trect;
           LargCarac,HautCarac,Marge : integer;
           quadrillage : boolean;
           canvas : Tcanvas;
           Xcroix,Ycroix : integer;
           ListeRepere : Tlist;
           Impression : boolean;
           Constructor Create;
           Destructor Destroy;override;
           Procedure Draw;
           Procedure TraceSpectre;
           Procedure TraceBeer;
           Procedure WindowXY(xr,yr : double;var xi,yi : integer);
           Procedure MondeXY(xi,yi : integer;var xr,yr : double);
           procedure ChercheMonde;
           procedure SetMonde(minX,minY,maxX,maxY : double);
           procedure DrawAxis;
//           Procedure TracePointSpectre(i : indiceVecteur);
           Procedure TracePointCinet(i : indiceVecteur);
           Procedure TracePointCalc(x,y : double);
           procedure SetHauteurFonte(H : integer);
           Function MaxiY : double;
           Function MaxiX : double;
           Procedure TraceCroix;
           Procedure AfficheCroix(x,y : double);
           Procedure TraceRepere(repere : Trepere);
           Procedure AjouteRepere(V : double;const T : shortString);
           Procedure Reset;
           Procedure Imprime;
  end;

var GrapheSpectre : Tgraphe;
    CouleurRepere : Tcolor = clRed;

IMPLEMENTATION

var MaxGr,MinGr : integer;

constructor Tmonde.Create;
begin
  inherited create;
	A := 1;
	B := 0;
	nom := '';
  unite := '';
	ZeroInclus := true;
	Maxi := 255;
	Mini := 0;
  PosCurseur := intSansSignif;
end;{--- Tmonde.Create ---}

procedure Tmonde.affecteEntier(r : double;var i : integer);
begin
      if r=sansSignif
         then i := intSansSignif
         else i := round(A*r+B)
end;

Procedure TGraphe.MondeXY(xi,yi : integer;var xr,yr : double);
begin
	xr := (xi-mondeX.B)/mondeX.A;
	yr := (yi-mondeY.B)/mondeY.A
end;

procedure Tgraphe.SetHauteurFonte(H : integer);
begin with canvas.Font do begin
     H := H * 72 div PixelsPerInch;
     H := 2 * (H div 2);
     if H<=8 then H := 8; // 8 points
     if H>=16 then H := 16; // 16 points
     Size := H;
     HautCarac := canvas.textHeight('A');
     Marge := hautCarac div 3;
     LargCarac := canvas.textWidth('A');
     if H>12
        then name := 'Arial Regressi'
        else name := 'Regressi';
end end;

Procedure TGraphe.WindowXY(xr,yr : double;var xi,yi : integer);
begin
        try
      	mondeX.affecteEntier(xr,xi);
      	mondeY.affecteEntier(yr,yi);
        if (yi>limiteFenetre.Bottom) or
           (yi<limiteFenetre.top) or
           (yr=sansSignif)
            then yi := IntSansSignif;
        except
           yi := IntSansSignif
        end;
end;

procedure TGraphe.ChercheMonde;

procedure chercheProjectionY;
begin with mondeY do begin
       A := (limiteCourbe.Top-limiteCourbe.Bottom)/(Maxi-Mini);
       B := limiteCourbe.Bottom-Mini*A;
       setDelta;
       isDate := false;
end end; { chercheProjectionY }

procedure chercheProjectionX;
begin with mondeX do begin
      A := (limiteCourbe.right-limiteCourbe.left)/(Maxi-Mini);
      B := limiteCourbe.left-Mini*A;
      setDelta;
      setDeltaDateTime;
end end;

begin { ChercheMonde }
     with limiteFenetre do begin
         mondeY.avecAxe := (bottom-top)>NbreGradMin*hautCarac*5 div 4;
         mondeX.avecAxe := (right-left)>NbreGradMin*4*largCarac;
         mondeY.grandAxe := (bottom-top)>NbreGraduations*hautCarac*5 div 4;
         mondeX.grandAxe := (right-left)>NbreGraduations*4*largCarac;
         MaxGr := 2*right;
         MinGr := -right;
     end;
     limiteCourbe := limiteFenetre;
     inc(limiteCourbe.left,marge);
     inc(limiteCourbe.top,marge);
     dec(limiteCourbe.bottom,marge);
     dec(limiteCourbe.right,marge);
     if mondeY.avecAxe then
        inc(limiteCourbe.left,LongNombreAxe*largCarac);
     if mondeX.avecAxe then
        dec(limiteCourbe.bottom,hautCarac);
     chercheProjectionX;
     chercheProjectionY;
end; { ChercheMonde }

procedure TGraphe.SetMonde(minX,minY,maxX,maxY : double);
begin
     mondeX.mini := minx;
     mondeX.maxi := maxx;
     mondeY.mini := miny;
     mondeY.maxi := maxy;
end; { SetMonde }

{
Procedure Tgraphe.TracePointSpectre(i : indiceVecteur);
var xi,yi : integer;
begin with canvas do begin
   pen.color := Lambda_color(i);
   pen.width := 1;
   pen.style := psSolid;
   brush.style := bsSolid;
   brush.color := canvas.pen.color;
   WindowXY(i,valeurY[i],xi,yi);
   if yi<>IntSansSignif then
      rectangle(xi-2,yi-2,xi+2,yi+2);
end end; // tracePointSpectre
}

Procedure Tgraphe.TracePointCinet(i : indiceVecteur);
var xi,yi : integer;
begin with canvas do begin
   pen.color := Lambda_color(i);
   pen.width := 1;
   pen.style := psSolid;
   brush.style := bsSolid;
   brush.color := canvas.pen.color;
   WindowXY(valeurX[i],valeurY[i],xi,yi);
   if (yi<>IntSansSignif) and
      (xi<>IntSansSignif) then
      rectangle(xi-2,yi-2,xi+2,yi+2);
end end; // tracePointCinet

procedure TGraphe.TraceSpectre;
var i : indiceVecteur;
    PointPrecedent : boolean;
    xi,yi : integer;
begin with canvas do begin
     pen.style := psSolid;
     pen.width := 2;
     PointPrecedent := false;
     for i := spectro.debutLambda to spectro.finLambda do
        if valeurY[i]<>sansSignif then begin
        WindowXY(i,valeurY[i],xi,yi);
        if yi=intSansSignif
           then pointPrecedent := false
           else begin
           pen.color := lambda_color(i);
           if PointPrecedent
               then LineTo(xi,yi)
               else begin
                  MoveTo(xi,yi);
                  pointPrecedent := true;
               end;
        end
     end;{for i}
end end;{ TraceSpectre }

procedure TGraphe.TraceBeer;
var i : integer;
    xi,yi : integer;
begin with canvas do begin
     brush.Style := bsSolid;
     brush.color := Lambda_color(1);
     pen.color := brush.color;     
     for i := 0 to pred(nbrePoints) do begin
        WindowXY(valeurX[i],valeurY[i],xi,yi);
        if (yi<>intSansSignif) and
           (xi<>intSansSignif)
           then rectangle(xi-2,yi-2,xi+2,yi+2);
     end;{for i}
end end;{ TraceBeer }

procedure TGraphe.Draw;
var c : integer;
begin with canvas do begin
     pen.color := CouleurGr;
     pen.mode := pmCopy;
     pen.style := psSolid;
     brush.style := bsSolid;
     brush.color := clWhite;
     FillRect(LimiteFenetre);
     pen.width := 1;
     font.Color := clBlack;
     with limiteFenetre do
        setHauteurFonte((bottom-top) div 30);
     chercheMonde;
     drawAxis;
     if trace=tCinet then TraceBeer else TraceSpectre;
     for c := 0 to pred(ListeRepere.count) do
         TraceRepere(ListeRepere[c]);
     if trace<>tCinet then begin
        WindowXY(600,0,Xcroix,Ycroix);
        TraceCroix; // trace
     end;
end end;

constructor TGraphe.Create;
begin
  inherited create;
  mondeX := Tmonde.create;
  mondeY := Tmonde.create;
  quadrillage := true;
  trace := tSpectre;
  Xcroix := 0;
  Ycroix := 0;
  ListeRepere := Tlist.Create;
end;

destructor TGraphe.Destroy;
begin
  mondeX.free;
  mondeY.free;
  ListeRepere.free;
  inherited destroy;
end;

procedure Tmonde.SetDelta;
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const coeff : array[0..5] of double = (0.1,0.2,0.5,1,2,5);
var  largeur : double;
     Exponent : integer;
     index : byte;
begin
      Exponent := Floor(Log10(Maxi-Mini));
      largeur := (maxi-mini)*dix(-exponent);
      index := 0;
      if grandAxe
        then while ( (largeur/coeff[index])>=NbreGraduations ) and (index<5) do inc(index)
        else while ( (largeur/coeff[index])>=NbreGradMin ) and (index<5) do inc(index);
      DeltaAxe := coeff[index]*dix(exponent);
      PointDebut := Ceil(mini/deltaAxe)*deltaAxe;
end;

Function Tmonde.FormatAxe(x : double) : strCourte;
begin
     if isDate
         then result := FormatDateTime(FormatDuree,Arrondi*round(x/Arrondi)/UnJour)
         else result := FloatToStrF(x,ffGeneral,3,0)
end;

Function Tgraphe.MaxiY : double;
var j : integer;
begin
     result := spectro.miniAbs;
     for j := 0 to maxVecteur do
         if result<valeurY[j] then result := valeurY[j];
     if result<spectro.maxiAbs/10
        then result := 1
        else if result>2*spectro.maxiAbs
            then result := spectro.maxiAbs
            else result := result*1.1;
end;

Procedure Tgraphe.TraceCroix;
const taille = 5;
begin
   canvas.pen.mode := pmNot;
   canvas.moveTo(Xcroix-taille,Ycroix);
   canvas.lineTo(Xcroix+taille,Ycroix);
   canvas.moveTo(Xcroix,Ycroix-taille);
   canvas.lineTo(Xcroix,Ycroix+taille);
end;

Procedure Tgraphe.AfficheCroix(x,y : double);
begin
   TraceCroix; // efface
   WindowXY(x,y,Xcroix,Ycroix);
   TraceCroix; // trace
end;

procedure Tgraphe.TraceRepere(repere : Trepere);
var x,y : integer;
begin with Canvas,repere do begin
      Pen.color := couleurRepere;
      Pen.Style := psDash;
      windowXY(valeur,0,x,y);
      MoveTo(x,LimiteCourbe.top);
      LineTo(x,LimiteCourbe.bottom);
      Pen.Style := psSolid;
end end;{ TraceRepere }

procedure Tgraphe.AjouteRepere(V : double;const T : shortString);
var repere : Trepere;
begin
     repere := Trepere.Create;
     repere.valeur := V;
     repere.text := T;
     ListeRepere.Add(repere);
     TraceRepere(repere);
end;

procedure TGraphe.Reset;
begin
     NbrePoints := 0;
     ListeRepere.Clear;
     NbreCourbe := 1;
end;

procedure Tgraphe.Imprime;
var sauveCanvas : Tcanvas;
begin
    if MessageDlg('Impression ?',mtConfirmation,mbOKCancel,0)=mrOK then begin
         sauveCanvas := canvas;
         printer.Orientation := poLandscape;
         printer.beginDoc;
         canvas := printer.canvas;
         limiteFenetre := canvas.clipRect;
         impression := true;
         draw;
         impression := false;
         printer.endDoc;
         canvas := SauveCanvas;
    end;
end;

Function Tgraphe.MaxiX : double;
var j : IndiceVecteur;
begin
     result := 0;
     for j := 0 to pred(NbrePoints) do
         if valeurX[j]>result then result := valeurX[j]
end;{maxiX}

procedure Tgraphe.TracePointCalc(x,y : double);
var xi,yi : integer;
begin with canvas do begin
      Pen.color := couleurCalc;
      Brush.color := couleurCalc;
      Brush.style := bsSolid;
      WindowXY(x,y,xi,yi);
      FillRect(rect(xi-dimPoint,yi-dimPoint,xi+dimPoint,yi+dimPoint));
end end;{ TracePointBeer }

Procedure Tmonde.setDeltaDateTime;
begin
        isDate := (unite='s') and (maxi>2*UneMinute);
        if not isDate then exit;
        Arrondi := 1;
        FormatDuree := 'hh:mm:ss';
        if maxi>UneHeure
             then begin
                     deltaAxe := QuartdHeure*ceil(maxi/6/QuartdHeure);
                     Arrondi := UneMinute;
             end
             else if maxi>QuartdHeure
                  then begin
                      deltaAxe := UneMinute*ceil(maxi/6/UneMinute);
                      Arrondi := 10;
                  end
                  else begin
                      deltaAxe := 10*ceil(maxi/UneMinute);
                      Arrondi := 1;
                  end;
        PointDebut := 0;
end; { setDeltaDateTime }

{$I axes}

end.

